import sys, time, pygame
from pygame.locals import *
import ftg_dataloader, ftg_frames, ftg_object

#level events
DEBUG_MODE=pygame.USEREVENT+2
TOGGLE_AUDIO=pygame.USEREVENT+3


class Level():
	def __init__(self, currentLevel, screenSize, inventory, sortedLayers=[]):
		self.currentLevel=currentLevel
		self.nextLevel=self.currentLevel+1
		self.quitGame=False
		self.screenSize=screenSize
		self.inventory=inventory
		self.levelCompleted=False
		self.levelPaused=False
		self.levelSprites=[]
		pygame.event.clear() #cleans any key pressure at level start
		self.debugOn=False

	def setBackground(self, filename):
		tmpBackgroundImage=ftg_frames.getImageFramesForAction(filename)[0]
		imageOriginalSize=tmpBackgroundImage.get_size()
		sizeRatio=min(float(self.screenSize[0])/imageOriginalSize[0],float(self.screenSize[1])/imageOriginalSize[1])
		if sizeRatio==1.0:
			self.backgroundImage=tmpBackgroundImage
		else:
			self.backgroundImage=pygame.transform.scale(tmpBackgroundImage,[int(sizeRatio*imageOriginalSize[0]), int(sizeRatio*imageOriginalSize[1])])

	def changeStatus(self,status=None):
		if status<>None:
			self.status=status
		else:
			self.status+=1
		self.timeStart=time.time()

	def eventsManagement(self,event):
		'''
		if event.type == DEBUG_MODE: #debug on/off
			self.debugOn = not self.debugOn			
		elif event.type == TOGGLE_AUDIO: #audio on/off
			if self.musicOn:
				pygame.mixer.music.stop()
			else:
				pygame.mixer.music.play()
			self.musicOn = not self.musicOn
		'''
		#check if key pressure is caught in Level subclasses
		keyPressureCaught=False
		if event.type in (KEYDOWN,KEYUP,MOUSEMOTION,MOUSEBUTTONUP,MOUSEBUTTONDOWN): #keys
			keyPressureCaught=self.checkKeyPressure(event)
		if not keyPressureCaught:
			if event.type == KEYDOWN:
				if event.key == K_ESCAPE: #quit
					self.saveInventory()
					self.quitGame=True
					#sys.exit('Pressed ESC')
				elif event.key == K_F12: #quit
					self.debugOn = not self.debugOn	
					self.printDebug('DEBUG '+str(self.debugOn), True)
				'''
				if self.debugOn:
					if event.key == K_BACKSPACE: #restart level
						self.nextLevel=self.currentLevel
						self.setLevelCompleted()
					if event.key == K_PAGEUP:
						self.jumpToLevel(self.currentLevel-1)
					if event.key == K_PAGEDOWN:
						self.jumpToLevel(self.currentLevel+1)
				'''
	def printDebug(self, message, always=False):
		if self.debugOn or always:
			print(str(message))
	
	def checkKeyPressure(self,keyEvent):
		#overwritten in subclasses
		return False
				
	def update(self):
		self.levelSprites.update() #sprite update
		self.customUpdate() #level mechanics
	
	def customUpdate(self):
		#overwritten in subclasses
		pass	

	def setLevelCompleted(self):
		self.levelCompleted=True
	
	def jumpToLevel(self, nextLevelNum):
		self.nextLevel=nextLevelNum
		self.setLevelCompleted()

	def jumpToNextLevel(self):
		self.jumpToLevel(self.currentLevel+1)

	def saveInventory(self):
		pass


class LevelWithFullScreenRefresh(Level):
	def __init__(self, currentLevel, screenSize, inventory):
		Level.__init__(self, currentLevel, screenSize, inventory)

	#adds specified object to levelSprites list setting proper layerOrder
	def addObject(self, newObj):
		self.levelSprites.append(newObj)
		return newObj 

	#remove specified object from levelSprites
	def delObject(self, delObj, lists=[]):
		try:
			self.levelSprites.remove(delObj)
		except:
			print '######## WTF? ###########'
		try:
			for l in lists:
			   l.remove(delObj)
		except:
			print '######## WTF? ###########'
		return None

	def update(self):
		for spr in self.levelSprites:
			spr.update()
		self.customUpdate()
			
class LevelWithLayeredUpdates(Level):
	def __init__(self, currentLevel, screenSize, inventory, sortedLayers=[]):
		Level.__init__(self, currentLevel, screenSize, inventory, sortedLayers)
		self.sortLayers(sortedLayers)
		self.levelSprites=pygame.sprite.LayeredDirty()

	#order of the objects layer, from back to front (last object will be always visible)
	def sortLayers(self, sortedLayers):
		self.layerOrder=sortedLayers

	#adds specified object to levelSprites list setting proper layerOrder
	def addObject(self, newObj):
		spriteLayer=0
		try:
			#print self.layerOrder, newObj.__class__.__name__
			spriteLayer=self.layerOrder.index(newObj.__class__.__name__)
		except ValueError:
			try:
				for parent in newObj.__class__.__bases__:
					spriteLayer=self.layerOrder.index(parent.__name__)
			except ValueError:
				spriteLayer=0
		
		self.levelSprites.add(newObj, layer=spriteLayer)
		#print 'add',newObj.__class__.__name__,len(self.levelSprites)		
		#print newObj,self.levelSprites.get_layer_of_sprite(newObj)
		return newObj 
	
	#remove specified object from levelSprites
	def delObject(self, delObj, lists=[]):
		try:
			self.levelSprites.remove(delObj)
		except:
			print '######## WTF? ###########'
		try:
			for l in lists:
			   l.remove(delObj)
		except:
			print '######## WTF? ###########'
		#print 'del',delObj.__class__.__name__,len(self.levelSprites)			
		return None
			
			
class Splash(LevelWithLayeredUpdates):
	def __init__(self, currentLevel, screenSize, inventory, logoImageName, textColors):
		LevelWithLayeredUpdates.__init__(self, currentLevel, screenSize, inventory)
		self.backgroundImage=pygame.Surface(screenSize)
		self.backgroundImage.fill(pygame.Color('Black'))
		logo = {'default':[ftg_dataloader.load_image(logoImageName)]}
		originalSize=logo['default'][0].get_size()
		logoSize=[int(originalSize[0]/(originalSize[1]/float(screenSize[1]))),screenSize[1]]
		'''
		logoPosition=[screenSize[0]/2,screenSize[1]/2]
		self.addObject(ftg_object.GameObject(logo, logoSize, logoPosition))
		'''
		self.addObject(ftg_object.Board(logo, logoSize, screenSize))
		self.textColors=textColors
		self.imageFileNames=ftg_dataloader.getImageFileNames()
		self.firstLoop=True
		
	def createInfoText(self, font):
		textPosition=[self.screenSize[0]/15,self.screenSize[1]*14/15]
		self.infoText=self.addObject(ftg_object.GameText(font,'Loading images...',textPosition,self.textColors[0]))
		
	def splitImageIntoFrames(self,fullImage,framesInaRowCol):
		framesInaCol,framesInaRow=framesInaRowCol
		frame_width = fullImage.get_width()/framesInaRow
		frame_height = fullImage.get_height()/framesInaCol
		
		animationTuple=[]
		for row in range(framesInaCol):
			for col in range(framesInaRow):
				singleFrame=fullImage.subsurface((col*frame_width,row*frame_height,frame_width,frame_height))
				animationTuple.append(singleFrame)
		return animationTuple
	
	def assignFramesToAction(self,imgName,animationTuple):
		actionAnimationDict={}
		try:
			for action in ftg_frames.getImageActionsNames(imgName):
				actionAnimationTuple=[]
				for f in ftg_frames.getImageFramesNumbers(imgName, action):
					actionAnimationTuple.append(animationTuple[f-1])
				actionAnimationDict[action]=actionAnimationTuple
		except IndexError:
			print imgName,action,'has',len(animationTuple),ftg_frames.getImageRowCol(imgName),'frames. frame #',(f-1),'has been assigned to action',action
			raise IndexError
		return actionAnimationDict
		
	def customUpdate(self):
		lastImageName=''
		if self.firstLoop==True: #first loop does nothing, just draws background
			self.firstLoop=False
		else:
			if len(self.imageFileNames)>0:
				imgName=self.imageFileNames.pop(0)
				print 'loading', imgName
				self.infoText.editText(imgName)
				self.infoText.setColor(self.textColors[len(self.imageFileNames)%len(self.textColors)])
				
				if imgName in ftg_frames.getImagesNames():
					img=ftg_dataloader.load_image(imgName)
					animationTuple=self.splitImageIntoFrames(img,ftg_frames.getImageRowCol(imgName))
					ftg_frames.allFramesLoaded[imgName]=self.assignFramesToAction(imgName,animationTuple)
				else:
					img=ftg_dataloader.load_image(imgName)
					ftg_frames.allFramesLoaded[imgName]={}
					ftg_frames.allFramesLoaded[imgName]['default']=[img]
				try:
					ftg_frames.allFramesLoaded[imgName]['row,col']=ftg_frames.allFramesDetails[imgName]['row,col']
				except KeyError:
					ftg_frames.allFramesLoaded[imgName]['row,col']=[1,1]
			else:
				self.jumpToNextLevel()
			


class Title(LevelWithLayeredUpdates):
	def __init__(self, currentLevel, screenSize, inventory, menuFont, menuText, menuColors, menuPosition, backgroundImageName):
		LevelWithLayeredUpdates.__init__(self, currentLevel, screenSize, inventory)	
		self.setBackground(backgroundImageName)
		self.createMenu(menuFont, menuText, menuColors, menuPosition)

	def createMenu(self, menuFont, menuText, menuColors, menuPosition):
		self.menuText=menuText
		self.menuColors=menuColors 
		self.menuValue=0
		self.menu=[]
		textHeight=menuFont.size('0')[1]
		for mt in range(len(self.menuText)):
			self.menu.append(self.addObject(ftg_object.GameText(menuFont, self.menuText[mt],[menuPosition[0],menuPosition[1]+textHeight*mt],self.menuColors[0])))
		self.menu[self.menuValue].setColor(self.menuColors[1])		
		self.updateMenu()

	def updateMenu(self):
		#update color
		for m in range(len(self.menu)):
			if m==self.menuValue:
				self.menu[m].setColor(self.menuColors[1])
				self.menu[m].dirty=1
			else:
				self.menu[m].setColor(self.menuColors[0])		 
				self.menu[m].dirty=1
	
	def checkKeyPressure(self,keyEvent):
		if keyEvent.type == KEYDOWN:
			if keyEvent.key == K_UP:
				self.menuValue=(self.menuValue-1)%len(self.menuText)
				self.updateMenu()
				return True
			elif keyEvent.key == K_DOWN:
				self.menuValue=(self.menuValue+1)%len(self.menuText)		
				self.updateMenu()
				return True
			elif keyEvent.key == K_RETURN:
				self.menuEntered(self.menu[self.menuValue].text)
				return True

	def menuEntered(self,value):
		print(value)
		self.updateMenu()
'''
class Credits(LevelWithFullScreenRefresh):
	def __init__(self, currentLevel, screenSize, debugOn, musicOn, inventory, textColors=[pygame.Color('Salmon'),pygame.Color('Red')], backgroundImageName='screen_credits.png', sortedLayers=[]):
		LevelWithFullScreenRefresh.__init__(self, currentLevel, screenSize, animations, debugOn, musicOn, inventory, sortedLayers)
		self.setBackground(backgroundImageName)
		self.createInfoText(textColors)

	def createInfoText(self, textColors):
		self.textColors=textColors
		textSize=self.screenSize[0]/20
		self.infoText=self.addObject(ftg_object.GameText('click to continue...',textSize,[self.screenSize[0]/16,self.screenSize[1]*11/12],self.textColors[0]))
		
	def checkKeyPressure(self,keyEvent):
		if keyEvent.type == KEYDOWN and keyEvent.key <> K_ESCAPE:
			self.setLevelCompleted()

		if keyEvent.type == MOUSEBUTTONDOWN:
			self.setLevelCompleted()
		return True
'''

class Intro(LevelWithFullScreenRefresh):
	def __init__(self, currentLevel, screenSize, inventory, storyLength, imageName):
		LevelWithFullScreenRefresh.__init__(self, currentLevel, screenSize, inventory)
		self.storyLength=storyLength
		self.storyIndex=1
		self.imageName=imageName
		self.setBackground('screen_'+self.imageName+str(self.storyIndex)+'.png')
		
  
	def checkKeyPressure(self,keyEvent):
		if keyEvent.type == KEYDOWN and keyEvent.key <> K_ESCAPE:
			if self.storyIndex<self.storyLength:
				self.storyIndex+=1
				self.setBackground('screen_'+self.imageName+str(self.storyIndex)+'.png')
			else:
				self.setLevelCompleted()
			return True


class LevelSelection(LevelWithLayeredUpdates):
	def __init__(self, currentLevel, screenSize, debugOn, musicOn, inventory, sortedLayers):
		#LevelWithFullScreenRefresh.__init__(self, currentLevel, screenSize, debugOn, musicOn, inventory)
		LevelWithLayeredUpdates.__init__(self, currentLevel, screenSize, debugOn, musicOn, inventory, sortedLayers)
		self.setBackground('screen_levels.png')

	def checkKeyPressure(self,keyEvent):
		if keyEvent.type == MOUSEBUTTONDOWN and keyEvent.button == 1:
			for lev in self.levelMiniatures:
				if lev.rect.collidepoint(pygame.mouse.get_pos()):
					if lev.onClick()>0:
						self.jumpToLevel(lev.onClick())
		elif keyEvent.type == KEYDOWN and keyEvent.key == K_ESCAPE:
			self.jumpToLevel(TITLESCREEN)
			return True