import ftg_frames
import pygame, copy
from pygame.locals import *
pygame.init()

class GameText(pygame.sprite.DirtySprite):  
	def __init__(self, font, text, topleft, color):
		pygame.sprite.DirtySprite.__init__(self)
		self.font=font
		self.text=text
		self.size=0
		self.initialtopleft=topleft
		self.color=color
		self.image=self.font.render(self.text, True, self.color).convert_alpha()

		self.rect=self.image.get_rect()
		self.rect.topleft=topleft
		self.scrollDelta=[0,0]
	
	def editText(self,newtext):
		self.text=str(newtext)
		self.dirty=1
		#self.update()
		
	def move(self,newtopleft):
		self.rect.topleft=newtopleft
		self.dirty=1
		
	def resize(self,newsize):
		self.size=newsize
		self.font=pygame.font.Font(myDataloader.load_font(CUSTOMFONT), self.size)
		self.dirty=1
		#self.update()
		
	def setColor(self,newcolor):
		self.color=newcolor
		self.dirty=1
		#self.update()
	
	def update(self):
		self.image=self.font.render(self.text, True, self.color).convert_alpha()
		
	   

class LevelTimer(GameText):
	def __init__(self, font, topleft, color, fps, startVal=0, delta=1, timerFormat='S'):
		GameText.__init__(self, font, '', topleft, color)
		self.fps=fps
		self.counter=(startVal+1)*60-1
		self.delta=delta
		self.timerFormat=timerFormat
		self.started=False

	def update(self):
		if self.started:
			self.counter+=1*self.delta
			self.text=self.timerFormat.replace('M',str((self.counter/self.fps)/60).rjust(2,'0')).replace('S',str((self.counter/self.fps)%60).rjust(2,'0')).replace('D',str(self.counter%self.fps).rjust(2,'0'))
			self.image=self.font.render(self.text, True, self.color).convert_alpha()
			self.dirty=1

	def start(self):
		self.started=True
	
	def reset(self):
		self.started=False
		self.counter=0
		self.dirty=1
		
	def restart(self):
		self.counter=0
		self.started=True

		
class MyLayeredUpdates(pygame.sprite.LayeredUpdates): 
	def __init__(self, *sprites, **kwargs):
		pygame.sprite.LayeredUpdates.__init__(self, *sprites, **kwargs)

	def draw(self, surface, scrollDelta=[0,0]):
		"""draw all sprites in the right order onto the passed surface.
		LayeredUpdates.draw(surface): return Rect_list
		"""
		spritedict = self.spritedict
		surface_blit = surface.blit
		dirty = self.lostsprites
		self.lostsprites = []
		dirty_append = dirty.append
		for spr in self.sprites():
			if scrollDelta==[0,0]:
				newrect = surface_blit(spr.image, spr.rect)
			else:
				newrect = surface_blit(spr.image, pygame.Rect([spr.rect.left-scrollDelta[0],spr.rect.top-scrollDelta[1]],spr.rect.size))
			rec = spritedict[spr]
			if rec is 0:
				dirty_append(newrect)
			else:
				if newrect.colliderect(rec):
					dirty_append(newrect.union(rec))
				else:
					dirty_append(newrect)
					dirty_append(rec)
			spritedict[spr] = newrect
		return dirty	
	
	
class GameObject(pygame.sprite.DirtySprite): 
	def __init__(self, image_name, size, centerPosition=None, firstFrame='default', rgb=[255,255,255]):
		pygame.sprite.DirtySprite.__init__(self)
		self.size=int(size[0]),int(size[1])
		if centerPosition==None: centerPosition=self.size
		self.topleft=[int(centerPosition[0]-size[0]/2),int(centerPosition[1]-size[1]/2)]
		
		if type(image_name)==str:
			self.images=ftg_frames.getImageFrames(image_name)
		elif type(image_name)==dict:
			#self.images=copy.deepcopy(image_name)
			self.images=self.createCopyOfImage(image_name)


		self.image=pygame.Surface((self.size))		 

		self.rect=self.image.get_rect()
		self.setPosition(centerPosition)
		#self.scrollDelta=[0,0]
		
		self.r,self.g,self.b=rgb
		self.animationTimer=0		
		self.lookingLeft=False
		self.baseAnimationName=firstFrame
		self.currentAnimationName=self.baseAnimationName
		self.currentAnimationRepeat=-1 #-1 forever
		self.currentAnimationIndex=0
		self.changeAnimation(self.baseAnimationName,-1,False)

		self.loadFrame()
		#full mask
		#self.mask=pygame.mask.from_surface(self.image) 
		
		#collision mask (only when image has alpha levels)
		self.collision_mask=pygame.mask.from_surface(self.image, 253)
		
		self.dirty=1

	def createCopyOfImage(self,images):
		return images

			
	'''
	# you can use alpha levels to customize collisions
	def getRect4Collisions(self):
		rectSize=10
		return pygame.Rect(self.rect.centerx-rectSize/2,self.rect.centery-rectSize/2,rectSize+1,rectSize+1)
	'''

	def setTopLeft(self,topleft):
		self.rect.topleft=topleft

	def getTopLeft(self):
		return self.rect.topleft
   
	def setPosition(self, center):
		self.rect.center=center
	
	def getPosition(self):
		return self.rect.center
	
	def getSize(self):
		return self.size
	
	def setSize(self, newSize):
		oldCenter=self.rect.center
		self.size=newSize
		self.image=pygame.Surface((self.size))		 
		self.rect=self.image.get_rect()
		self.loadFrame()
		self.setPosition(oldCenter)
		
	def getAllFrames(self):
		frames=[]
		for k in self.images.keys():
			if k<>'row,col':
				for i in self.images[k]:
					frames.append(i)
		return frames

	def changeColorToAllFrames(self,fromColor,toColor):
		for i in self.getAllFrames():
			i = pygame.PixelArray(i)
			tolerance=0.1
			i.replace(fromColor, toColor, tolerance)
	
	def move(self):
		pass
				
	def loadFrame(self):
		try:
			self.image=pygame.transform.scale(self.images[self.currentAnimationName][self.currentAnimationIndex], self.size)
			if self.lookingLeft:
				self.image=pygame.transform.flip(self.image, True, False)
			self.image=self.get_alpha_surface(self.image, 255, self.r, self.g, self.b)
			self.dirty=1
		except:
			raise			

			
	def changeAnimation(self, name, repeat=-1, stopRepeating=True, forceImmediate=False):
		self.nextAnimationName=name
		self.nextAnimationRepeat=repeat
		if forceImmediate:
			self.currentAnimationRepeat=0
			self.currentAnimationIndex=0
		elif stopRepeating:
			self.currentAnimationRepeat=0
		

	def changeFrame(self):
		numOfAnim=len(self.images[self.currentAnimationName])
		self.currentAnimationIndex=(self.currentAnimationIndex+1)%numOfAnim
		self.loadFrame()

	def updateAnimation(self):
		if self.currentAnimationIndex>=len(self.images[self.currentAnimationName])-1: #end of animation loop
			if self.currentAnimationRepeat<=0: #change animation
				self.currentAnimationName=self.nextAnimationName
				self.currentAnimationRepeat=self.nextAnimationRepeat
			else:
				self.currentAnimationRepeat-=1 #decrease one repetition
			#in any case increase index and change frame
			self.changeFrame()
		elif len(self.images[self.currentAnimationName])>1: #animation loop > 1 --> change frame in the same animation
			self.changeFrame()

			
		

		
	def get_alpha_surface(self, surf, alpha=128, red=128, green=128, blue=128, mode=pygame.BLEND_RGBA_MULT):
		"""returns a copy of a surface object with user-defined 
		   values for red, green, blue and alpha. 
		   Values from 0-255. 
		   thanks to Claudio Canepa <ccanepacc@gmail.com>
		   for this function."""
		tmp = pygame.Surface(surf.get_size(), pygame.SRCALPHA, 32)
		tmp.fill( (red,green,blue,alpha) )
		tmp.blit(surf, (0,0), surf.get_rect(), mode)
		return tmp		
					
	def update(self):
		self.animationTimer=(self.animationTimer+1)%ftg_frames.FPS_FRAME
		if self.animationTimer==0:
			self.updateAnimation()
		self.updatePosition()
		
	def updatePosition(self):
		pass	
		
class GameObjectReusable(GameObject): 
	def __init__(self, images, size, centerPosition, rgb=[255,255,255]):
		GameObject.__init__(self, images, size, centerPosition, rgb=[255,255,255])

	def createCopyOfImage(self,images):
		copiedImages={}
		for ik in images.keys():
			if ik<>'row,col':
				copiedImages[ik]=[]
				for s in images[ik]:
					copiedImages[ik].append(s.copy())
		copiedImages['row,col']=images['row,col']
		return copiedImages

	
class Board(GameObject):	
	def __init__(self, images, size, screensize, alignment='center'):
		if alignment=='center':
			centerPosition=[s/2 for s in screensize]
		elif alignment=='left':
			centerPosition=[screensize[0]/4, screensize[1]/2]
		elif alignment=='right':
			centerPosition=[screensize[0]*3/4, screensize[1]/2]			
		GameObject.__init__(self, images, size, centerPosition)
	
	
	