import sys, time, pygame, random, math ,re
from pygame.locals import *

import ftg_dataloader, ftg_level, ftg_object, ftg_frames
import pw16_object as customObject
from pw16_variable import *
from collections import Counter



def createLevel(screen,nextLevel,animations,inventory):
	screenSize=(screen.get_width(),screen.get_height())
	sortedLayers=['Toolbar','Ring','Combo','ComboElement','Room','Bubble','Enemy','RoomObject','You','Message','Heart','GameText','PauseScreen','MouseCursor']
	#splashscreen	
	if nextLevel==SPLASHSCREEN:		
		currentLevel=Splash(nextLevel,screenSize, inventory)	
	else:
		#start music
		if not pygame.mixer.music.get_busy() and MUSIC_FLAG:
				pygame.mixer.music.play(-1)		  
		#choose level
		if nextLevel==GAMEOVER:
			storyLength=1
			currentLevel=Gameover(nextLevel,screenSize,inventory)
		elif nextLevel==GAMEINTRO:
			storyLength=5
			currentLevel=ftg_level.Intro(nextLevel,screenSize,inventory,storyLength,'intro')
		elif  nextLevel==NEMESIS:
			currentLevel=Nemesis(nextLevel,screenSize,inventory,sortedLayers)
		elif nextLevel==YOUWIN:
			storyLength=1
			currentLevel=Youwin(nextLevel,screenSize,inventory)
		elif nextLevel==TITLESCREEN:
			currentLevel=Title(nextLevel, screenSize, inventory)
		else:
			print(('WARNING: level {levnum} does not exist').format(levnum=nextLevel))
			nextLevel=TITLESCREEN
			currentLevel=Title(nextLevel, screenSize, inventory)
	return currentLevel	

	
	
	
class Splash(ftg_level.Splash):
	def __init__(self, currentLevel, screenSize, inventory):
		logoImageName='screen_forkedtaillogo.png'
		textColors=[pygame.Color('Purple')]
		ftg_level.Splash.__init__(self, currentLevel, screenSize, inventory, logoImageName, textColors)
		self.createInfoText(NORMAL_CUSTOMFONT)
		
		
class Title(ftg_level.Title):
	def __init__(self, currentLevel, screenSize, inventory):
		menuFont=NORMAL_CUSTOMFONT
		menuText=['play','quit']
		menuColors=[pygame.Color('White'),pygame.Color('Grey')]
		menuPosition=(screenSize[0]/2,screenSize[1]*3/4)
		backgroundImageName='screen_title1.png'		
		ftg_level.Title.__init__(self, currentLevel, screenSize, inventory, menuFont, menuText, menuColors, menuPosition, backgroundImageName)

	def menuEntered(self,value):
		#menuText=['play','audio '+(not MUSIC_FLAG)*'on'+(MUSIC_FLAG)*'off','credits','quit']
		if value=='play':
			self.setLevelCompleted()
			self.jumpToLevel(GAMEINTRO)
		elif value=='quit':
			sys.exit('quit from menu')							
		self.updateMenu()

class Credits(ftg_level.Intro):
	def __init__(self, currentLevel, screenSize, inventory):
		storyLength=1
		imageName='credits'
		ftg_level.Intro.__init__(self, currentLevel, screenSize, inventory, storyLength, imageName)


class Gameover(ftg_level.Intro):	
	def __init__(self, currentLevel, screenSize, inventory):
		storyLength=1
		imageName='gameover'
		ftg_level.Intro.__init__(self, currentLevel, screenSize, inventory, storyLength, imageName)
		
		
class Youwin(ftg_level.Intro):	
	def __init__(self, currentLevel, screenSize, inventory):
		storyLength=1
		imageName='youwin'
		ftg_level.Intro.__init__(self, currentLevel, screenSize, inventory, storyLength, imageName)






class Nemesis(ftg_level.LevelWithLayeredUpdates):
	def __init__(self, currentLevel, screenSize, inventory, sortedLayers):
		ftg_level.LevelWithLayeredUpdates.__init__(self, currentLevel, screenSize, inventory, sortedLayers)

		self.backgroundImage=pygame.Surface(screenSize,pygame.SRCALPHA, 32).convert_alpha()
		self.backgroundImage.fill(pygame.Color('Black'))
		self.setBackground('screen_background1.jpg')
		self.pauseScreen=None
		self.resultScreen=None
		self.loseScreen=None
		self.helpScreen=None
		self.helpScreen=self.addObject(customObject.PauseScreen(ftg_frames.getImageFrames('screen_pause1.png'),self.screenSize))
		
		self.clock=0
		self.speed=4*FPS
		


		self.numOfRooms=[10,10]  #[x, y]=x rooms in a row, y rooms in a col
		self.numOfCells=[15,10] #[x, y]=x cells in a row, y cells in a col [for every room]
		self.printDebug('self.numOfCells {xy}'.format(xy=self.numOfCells))
		#self.cellSize=[int(self.screenSize[0]/(self.numOfCells[0]+1)),int(self.screenSize[1]/(self.numOfCells[1]+3))]
		self.cellSize=2*[int(self.screenSize[1]/(self.numOfCells[1]+3))]
		self.printDebug('self.cellSize {cs}'.format(cs=self.cellSize))
		self.toolBarSize=[self.screenSize[0],2*self.cellSize[1]]
		self.printDebug('self.toolBarSize {tbs}'.format(tbs=self.toolBarSize))
		sparePixel=[(self.screenSize[0]-self.cellSize[0]*self.numOfCells[0])/2,(self.screenSize[1]-self.toolBarSize[1]-self.cellSize[1]*self.numOfCells[1])/2]
		self.printDebug('sparePixel {sp}'.format(sp=sparePixel))
		self.boardLimits=[[sparePixel[0],self.toolBarSize[1]+sparePixel[1]],[self.screenSize[0]-sparePixel[0],self.screenSize[1]-sparePixel[1]]]
		self.printDebug('self.boardLimits {bl}'.format(bl=self.boardLimits))
		self.mapCells=[]
		self.board=None
		self.yourBullets=[]
		#self.nextTo=None
		self.comboActive=False
		self.reading=None
		self.dying=[]
		self.sirene=[]
		
		self.currentRoom=self.inventory['yourPos'][0] #x_room,y_room
		print 'self.currentRoom',self.currentRoom
		self.loadRoomBack()
		self.loadMap()
				
		
		#generate toolbar
		self.toolbar=self.addObject(customObject.Toolbar(ftg_frames.getImageFrames('obj_toolbar.png'),[self.screenSize[0],self.screenSize[1]/600.0*100],[self.screenSize[0]/2,50]))
		self.rings=[]
		ringSize=self.toolbar.rect.width/10
		ringStartPos=self.toolbar.rect.width/40
		for r in range(8):
			self.rings.append(self.addObject(customObject.Ring(ftg_frames.getImageFrames('obj_ring.png'),2*[ringSize*0.9],[ringStartPos+(r+1+r/4)*ringSize,50])))
			if self.inventory['elements'][r]:
				self.addObject(customObject.ComboElement(ftg_frames.getImageFrames('obj_elements.png'),2*[ringSize*0.7],ELEMENTS[r]))
				self.addObject(ftg_object.GameText(SMALL_CUSTOMFONT,str(r+1),[2*ringStartPos+(r+1+r/4)*ringSize,50],pygame.Color('White')))	
		self.rings.append(self.addObject(customObject.Ring(ftg_frames.getImageFrames('obj_ring.png'),2*[ringSize*1.1],[ringStartPos+5*ringSize,75])))
		self.combo=None
		self.comboText=None
		
		#generate you
		self.yourSize=self.cellSize
		self.yourPosition=[self.boardLimits[0][0]+(self.inventory['yourPos'][1][1]+0.5)*self.cellSize[1],self.boardLimits[0][1]+(self.inventory['yourPos'][1][0]+0.5)*self.cellSize[0]]
		#				[(obj.getCenterY()-self.boardLimits[0][1])/self.cellSize[1],(obj.getCenterX()-self.boardLimits[0][0])/self.cellSize[0]]
		#print 'yourPosition',self.inventory['yourPos'],'-->',self.yourPosition
		#print self.boardLimits[0][1],(self.inventory['yourPos'][1][1]+0.5),self.cellSize[1]
		
		self.you=self.addObject(customObject.You(ftg_frames.getImageFrames('obj_n_nemesis.png'),self.yourSize,self.yourPosition,self.boardLimits))
		self.yourSpeed=[c/10 for c in self.cellSize]
		
		#generate life
		lifePos=[self.boardLimits[1][0]+self.cellSize[0]/2,self.boardLimits[1][1]-self.cellSize[1]/2]
		self.addObject(customObject.Heart(ftg_frames.getImageFrames('obj_heart.png'),self.cellSize,lifePos))
		self.lifeText=self.addObject(ftg_object.GameText(NORMAL_CUSTOMFONT,str(self.you.life),[lifePos[0],lifePos[1]-self.cellSize[1]/3],pygame.Color('Black')))	
		
		#generate mouse cursor
		self.mouseCursor=self.addObject(customObject.MouseCursor(ftg_frames.getImageFrames('obj_mouse.png'),self.cellSize,pygame.mouse.get_pos()))
	

		
		#generate info text
		#self.infoText=self.addObject(ftg_object.GameText(' Level '+str(self.currentLevel)+' - Gold='+str(self.you.gold)+'/'+str(self.minCoinsToWin)+self.debugOn*('/'+str(self.inventory['level'][self.currentLevel]['max'])),25,(self.yourCollisionRect[0][0],0),pygame.Color('Purple')))
		#self.infoText=self.addObject(ftg_object.GameText(NORMAL_CUSTOMFONT,' Level '+str(self.currentLevel),(self.boardLimits[0][0],0),pygame.Color('Purple')))	
	
		self.selectedElements=[None,None,False]



	def loadRoomBack(self):
		filename='rooms.map'
		f = ftg_dataloader.load_level(filename)
		self.roomBack = f.readlines()
		f.close()
		
	def loadMap(self):
		filename='levels.map'
		f = ftg_dataloader.load_level(filename)
		self.map = f.readlines()
		f.close()
		checkMap=''.join(self.map)
		#if 'S' not in checkMap: self.printDebug('MAP CHECK ERROR: missing Start')
		#if 'F' not in checkMap: self.printDebug('MAP CHECK ERROR: missing Finish')
		self.generateRoom(self.currentRoom)
			
		
		
	def generateRoom(self, yx_room, yourPosition=None):
		y_room, x_room=yx_room
		#self.printDebug('deleting cells')
		[self.delObject(h) for c in self.mapCells for h in c]
		for b in self.yourBullets:
			self.delObject(b)
		self.yourBullets=[]
		self.delObject(self.board)
		#self.printDebug('deleted cells')
		
		#generate board
		if [y_room,x_room] in BLACKROOMS and (not self.comboActive or self.combo.name<>'torch'):
			currentPos=self.getCellFromPos(self.you)[0]
			self.mapCells=[]
			for line in range(self.numOfCells[1]):
				mapLine=[]
				for cell in range(self.numOfCells[0]):
					if [line,cell]==currentPos:
						mapLine.append(customObject.CellEmpty())
					else:
						mapLine.append(customObject.CellBlock())
						
				self.mapCells.append(mapLine)
			self.printDebug('num of objects on screen'+str(len(self.levelSprites))+str(Counter([ls.__class__.__name__ for ls in self.levelSprites])))
			return			
		
		if self.roomBack[y_room][x_room]=='x':
			filename='obj_room_'+str(y_room)+str(x_room)+'.png'
		else:
			filename='obj_room_'+self.roomBack[y_room][x_room]+'.png'
		self.board=self.addObject(customObject.Room(ftg_frames.getImageFrames(filename),[self.boardLimits[1][0]-self.boardLimits[0][0],self.boardLimits[1][1]-self.boardLimits[0][1]],[self.screenSize[0]/2,self.toolBarSize[1]/2+self.screenSize[1]/2]))

		mapByRow=self.map[y_room*self.numOfCells[1]+y_room:(y_room+1)*self.numOfCells[1]+y_room]
		mapByCol=[]
		for line in mapByRow:
			mapByCol.append(line[x_room*self.numOfCells[0]+x_room:(x_room+1)*self.numOfCells[0]+x_room])
		
		self.mapCells=[]
		mapLine=[]
		row=column=0
		for line in mapByCol:
			for cell in line:
				if cell in ['\n','\r',os.linesep]:
					continue
				cellPosition=[self.boardLimits[0][0]+(column+0.5)*self.cellSize[0],self.boardLimits[0][1]+(row+0.5)*self.cellSize[1]]
				if cell =='n':
					self.yourPosition=cellPosition
					mapLine.append(customObject.CellEmpty())
				elif cell=='a':
					mapLine.append(self.addObject(customObject.Statue(ftg_frames.getImageFrames('obj_a_statue.png'),self.cellSize,cellPosition,self.currentRoom)))
				elif cell=='b':
					mapLine.append(self.addObject(customObject.Bush(ftg_frames.getImageFrames('obj_b_bush.png'),self.cellSize,cellPosition)))
				elif cell=='c':
					mapLine.append(self.addObject(customObject.Cupido(ftg_frames.getImageFrames('obj_c_cupido.png'),self.cellSize,cellPosition)))			
				elif cell=='E':
					self.doorHell=self.addObject(customObject.DoorHellImage(ftg_frames.getImageFrames('obj_d_doorhell.png'),[c*4 for c in self.cellSize],cellPosition,cell))
					mapLine.append(self.doorHell)						
				elif cell=='e':
					mapLine.append(self.addObject(customObject.Ecatonchiri(ftg_frames.getImageFrames('obj_e_ecatonchiri.png'),[2*c for c in self.cellSize],[cellPosition[0],cellPosition[1]-self.cellSize[1]/2])))						
				elif cell=='f':
					mapLine.append(self.addObject(customObject.Flower(ftg_frames.getImageFrames('obj_f_flower.png'),self.cellSize,cellPosition)))	
				elif cell=='g':
					mapLine.append(self.addObject(customObject.Soul(ftg_frames.getImageFrames('obj_g_soul.png'),self.cellSize,cellPosition)))						
				elif cell=='h':
					mapLine.append(self.addObject(customObject.Torch(ftg_frames.getImageFrames('obj_h_torch.png'),self.cellSize,cellPosition)))	
				elif cell=='i':
					mapLine.append(self.addObject(customObject.Cerbero(ftg_frames.getImageFrames('obj_i_cerbero.png'),[2*c for c in self.cellSize],[cellPosition[0],cellPosition[1]-self.cellSize[1]/2])))
				elif cell=='j':
					mapLine.append(self.addObject(customObject.Arpia(ftg_frames.getImageFrames('obj_j_arpia.png'),self.cellSize,cellPosition)))
				elif cell=='k':
					mapLine.append(self.addObject(customObject.Centauro(ftg_frames.getImageFrames('obj_k_centauro.png'),self.cellSize,cellPosition)))					
				elif cell=='l':
					mapLine.append(self.addObject(customObject.Lotofago(ftg_frames.getImageFrames('obj_l_lotofago.png'),self.cellSize,cellPosition)))
				elif cell=='m':
					self.sirene.append(self.addObject(customObject.Sirena(ftg_frames.getImageFrames('obj_m_sirena.png'),self.cellSize,cellPosition)))
					mapLine.append(self.sirene[-1])						
				elif cell=='o':
					free=self.currentRoom in self.inventory['owl']
					mapLine.append(self.addObject(customObject.Owl(ftg_frames.getImageFrames('obj_o_owl.png'),self.cellSize,cellPosition,free)))	
				elif cell=='p':
					mapLine.append(self.addObject(customObject.Pot(ftg_frames.getImageFrames('obj_p_pot.png'),self.cellSize,cellPosition)))		
				elif cell=='q':
					mapLine.append(self.addObject(customObject.PotFake(ftg_frames.getImageFrames('obj_q_potfake.png'),self.cellSize,cellPosition)))					
				elif cell=='Q':
					mapLine.append(self.addObject(customObject.Cavemonster(ftg_frames.getImageFrames('obj_q_cave_monster.png'),self.cellSize,cellPosition)))						
				elif cell=='r':
					mapLine.append(self.addObject(customObject.Rock(ftg_frames.getImageFrames('obj_r_rock.png'),self.cellSize,cellPosition)))
				elif cell=='s':
					mapLine.append(self.addObject(customObject.Satiro(ftg_frames.getImageFrames('obj_s_satiro.png'),self.cellSize,cellPosition)))						
				elif cell=='t':
					mapLine.append(self.addObject(customObject.Tree(ftg_frames.getImageFrames('obj_t_tree.png'),self.cellSize,cellPosition)))
				elif cell=='T':				
					mapLine.append(self.addObject(customObject.Tip(ftg_frames.getImageFrames('obj_t_tip.png'),self.cellSize,cellPosition)))	
				elif cell=='v':
					mapLine.append(self.addObject(customObject.Bat(ftg_frames.getImageFrames('obj_v_bat.png'),self.cellSize,cellPosition)))
				elif cell=='w':
					mapLine.append(self.addObject(customObject.Minotauro(ftg_frames.getImageFrames('obj_w_minotauro.png'),self.cellSize,cellPosition)))		
				elif cell=='z':
					mapLine.append(self.addObject(customObject.Ade(ftg_frames.getImageFrames('obj_z_ade.png'),[2*c for c in self.cellSize],[cellPosition[0],cellPosition[1]-self.cellSize[1]/2])))
				elif cell=='R':
					mapLine.append(self.addObject(customObject.DoorRed(ftg_frames.getImageFrames('obj_RY_door.png'),self.cellSize,cellPosition,cell)))
				elif cell=='Y':
					mapLine.append(self.addObject(customObject.DoorYellow(ftg_frames.getImageFrames('obj_RY_door.png'),self.cellSize,cellPosition,cell)))					
				elif cell=='A':
					mapLine.append(self.addObject(customObject.Amaltea(ftg_frames.getImageFrames('obj_a_amaltea.png'),self.cellSize,cellPosition)))					
				elif cell=='x':
					mapLine.append(customObject.CellBlock())
				elif cell=='C':
					mapLine.append(customObject.Charon())				
				elif cell=='D':
					mapLine.append(customObject.DoorHell())		
				elif cell=='H':
					mapLine.append(customObject.Hell())		
				elif cell in ('12345678'):
					if not self.inventory['elements'][int(cell)-1]:
						mapLine.append(self.addObject(customObject.Element(ftg_frames.getImageFrames('obj_elements.png'),self.cellSize,cellPosition,cell)))
						#mapLine.append(self.addObject(customObject.Bubble(ftg_frames.getImageFrames('obj_u_bubble.png'),[self.cellSize[0],self.cellSize[0]*2],cellPosition)))	
					else:
						mapLine.append(customObject.CellEmpty())
				else:
					mapLine.append(customObject.CellEmpty())
			
				column+=1
			self.mapCells.append(mapLine)
			row+=1
			mapLine=[]
			column=0					
		'''
		obstacles=self.board.getAlpha()
		for y in range(self.numOfCells[0])
			for x in range(self.numOfCells[1]):
				if obstacles[x][y]: self.mapCells[x][y].block=True
		'''

		self.printDebug('num of objects on screen'+str(len(self.levelSprites))+str(Counter([ls.__class__.__name__ for ls in self.levelSprites])))
			


	def checkKeyPressure(self,keyEvent):
		self.delObject(self.reading)
		#on pause
		if self.pauseScreen<>None:
			if (keyEvent.type == KEYDOWN and keyEvent.key == K_ESCAPE):
				self.jumpToLevel(TITLESCREEN)
				return True
			elif (keyEvent.type == KEYDOWN or keyEvent.type == MOUSEBUTTONDOWN):
				if (keyEvent.type == KEYDOWN and keyEvent.key in customVariables.help_key):
					self.helpScreen=self.addObject(customObject.PauseScreen(ftg_frames.getImageFrames('screen_pause1.png'),self.screenSize))
				self.delObject(self.pauseScreen)
				self.pauseScreen=None
				self.printDebug('Pause OFF',True)
		#on result
		elif self.resultScreen<>None:
			if (keyEvent.type == KEYDOWN and keyEvent.key == K_ESCAPE):
				self.jumpToLevel(TITLESCREEN)
				return True
			elif ((keyEvent.type == KEYDOWN and keyEvent.key == K_RETURN) or keyEvent.type == MOUSEBUTTONDOWN):
				self.delObject(self.resultScreen)
				self.resultScreen=None
				#nextLevel has already been calculated properly (same level or next level) in self.resultScreen.checkWinCondition()
				self.jumpToLevel(self.nextLevel)
		#on lose
		elif self.loseScreen<>None:
			if (keyEvent.type == KEYDOWN and keyEvent.key == K_ESCAPE):
				self.jumpToLevel(TITLESCREEN)
				return True
			elif (keyEvent.type == KEYDOWN or keyEvent.type == MOUSEBUTTONDOWN):
				self.delObject(self.loseScreen)
				self.loseScreen=None
				self.jumpToLevel(self.currentLevel)
		#on help
		elif self.helpScreen<>None:
			if (keyEvent.type == KEYDOWN and keyEvent.key == K_ESCAPE):
				self.jumpToLevel(TITLESCREEN)
				return True
			elif (keyEvent.type == KEYDOWN or keyEvent.type == MOUSEBUTTONDOWN):
				self.delObject(self.helpScreen)
				self.helpScreen=None
		#on game
		elif keyEvent.type == KEYDOWN:
			#cheat/debug begin
			'''
			if self.debugOn:
				if keyEvent.key == K_PAGEUP:
					self.jumpToLevel(self.currentLevel-1)
				if keyEvent.key == K_PAGEDOWN:
					self.jumpToLevel(self.currentLevel+1)
			'''
			#cheat/debug end

			if keyEvent.key == K_ESCAPE:
				self.helpScreen=self.addObject(customObject.PauseScreen(ftg_frames.getImageFrames('screen_pause1.png'),self.screenSize))
				self.printDebug('Pause ON', True)
				return True
			#movements start
			
			elif keyEvent.key in MOVE_LEFT:
				self.moveNemesis(-self.cellSize[0]/2+1,0)
			elif keyEvent.key in MOVE_RIGHT:
				self.moveNemesis(self.cellSize[0]/2+1,0)
			elif keyEvent.key in MOVE_UP:
				self.moveNemesis(0,-self.cellSize[1]/2+1)
			elif keyEvent.key in MOVE_DOWN:
				self.moveNemesis(0,self.cellSize[1]/2+1)
				
			elif keyEvent.key in ACTION:
				self.doAction()
			
			elif keyEvent.key in COMBINE:
				self.selectCombo(int(pygame.key.name(keyEvent.key))-1)
			elif keyEvent.key in EXECUTE:
				self.activateCombo()		
			#movements end
			
			#elif keyEvent.key in LEVEL_RESTART:
			#	self.jumpToLevel(self.currentLevel)
			'''
			elif keyEvent.key in HELP_KEY:
				self.helpScreen=self.addObject(customObject.PauseScreen(ftg_frames.getImageFrames('screen_pause1.png'),self.screenSize))
			'''	
		elif keyEvent.type == KEYUP:
			if keyEvent.key in MOVE_LEFT+MOVE_RIGHT+MOVE_UP+MOVE_DOWN:
				currentPos,nextPos=self.getCellFromPos(self.you,self.you.speed)
				#self.you.stop([(self.you.getCenterY()-self.boardLimits[0][1])/self.cellSize[1],(self.you.getCenterX()-self.boardLimits[0][0])/self.cellSize[0]])
				self.you.stop(nextPos)
				self.printDebug('KEYUP nextTo'+str(self.you.nextTo)+self.getObjFromCell(self.you.nextTo).__class__.__name__)
				'''
				if self.you.nextTo<>None:
					try:
						
						self.printDebug('KEYUP nextTo'+str(self.mapCells[self.you.nextTo[0]][self.you.nextTo[1]].__class__.__name__))
					except:
						pass
				'''
				
		elif keyEvent.type == MOUSEBUTTONDOWN:
			if keyEvent.button == 1:
				for r in self.rings:
					if r.rect.collidepoint(pygame.mouse.get_pos()):
						#r.changeAnimation('selected')
						i=self.rings.index(r)
						if i==8:
							self.activateCombo()
						else:
							self.selectCombo(i)
		return False		

	def moveNemesis(self, dirx, diry):
		if dirx<>0 or diry<>0:
			self.you.changeAnimationDir(dirx, diry)
		nextCellCollision=True
		try:
			currentPos,nextPos=self.getCellFromPos(self.you,[dirx, diry])
			#nextPos=[(self.you.getCenterY()-self.boardLimits[0][1]+diry)/self.cellSize[1],(self.you.getCenterX()-self.boardLimits[0][0]+dirx)/self.cellSize[0]]
			self.printDebug(str(currentPos)+'-->'+str(nextPos))
			#nextCell=self.mapCells[nextPos[0]][nextPos[1]]
			nextCell=self.getObjFromCell(nextPos)
			if nextCell.__class__==customObject.CellEmpty or nextCell.block==False:
				nextCellCollision=False			
		except IndexError:
			self.printDebug('ERROR on y'+str(self.you.getCenterY())+' x'+str(self.you.getCenterX()),True)
			nextCellCollision=False
		if nextCellCollision and not self.debugOn:
			self.you.stop(nextPos)
		else:
			if dirx<>0:
				self.you.move(int(math.copysign(self.yourSpeed[0],dirx)), 0)
			elif diry<>0:
				self.you.move(0, int(math.copysign(self.yourSpeed[1],diry)))

		if self.you.nextTo<>None:
			try:
				self.printDebug('moveNemesis nextTo'+str(self.mapCells[self.you.nextTo[0]][self.you.nextTo[1]].__class__.__name__))
			except IndexError:
				self.printDebug('moveNemesis nextTo'+str(self.you.nextTo))
			
	def getCellFromPos(self, obj, dir=[0,0]):
		dirx,diry=dir
		currentCell=[(obj.getCenterY()-self.boardLimits[0][1])/self.cellSize[1],(obj.getCenterX()-self.boardLimits[0][0])/self.cellSize[0]]
		nextCell=[(obj.getCenterY()-self.boardLimits[0][1]+diry)/self.cellSize[1],(obj.getCenterX()-self.boardLimits[0][0]+dirx)/self.cellSize[0]]
		return [currentCell,nextCell]
		
	def getObjFromCell(self, cell):
		if cell<>None:
			try:
				return self.mapCells[cell[0]][cell[1]]
			except IndexError:
				self.printDebug('ERROR on'+str(cell),True)
				return customObject.CellEmpty()
				
	
	def selectCombo(self, i):
		if self.inventory['elements'][i]:
			self.popMessage('This is '+ELEMENTS[i]+' from '+MUSES[i])
			#deactivate other rings
			for r in self.rings[i/4*4:(i/4+1)*4]:
				r.changeAnimation('default')
				self.rings[-1].changeAnimation('default')
			self.rings[i].changeAnimation('selected')
			self.selectedElements[i/4]=i%4
			self.selectedElements[2]=None

			self.delObject(self.combo)
			self.delObject(self.comboText)
			self.comboActive=False
			#restore speed
			self.yourSpeed=[c/10 for c in self.cellSize]
			#restore animation
			self.you.changeAction('default')

		
	def activateCombo(self):
		if None not in self.selectedElements[:2]:
			self.popMessage('You just activated '+COMBOS[self.selectedElements[0]][self.selectedElements[1]])
			for r in self.rings[:8]:
				r.changeAnimation('default')
			self.rings[-1].changeAnimation('selected')
			self.selectedElements[2]=True
			comboName=COMBOS[self.selectedElements[0]][self.selectedElements[1]]
			self.printDebug(comboName+' activated')
			self.combo=self.addObject(customObject.Combo(ftg_frames.getImageFrames('obj_combos.png'),2*[80*1.1],[20+5*80,75],comboName,str(self.selectedElements[0])+str(self.selectedElements[1])))
			self.comboText=self.addObject(ftg_object.GameText(SMALL_CUSTOMFONT,' ',[2*20+5*80,90],pygame.Color('White')))	
			self.comboActive=True
			self.selectedElements[0]=self.selectedElements[1]=None
			if comboName in ['torch','shield','immune']:
				self.you.changeAction(comboName)
			#else:
				
		
	def doAction(self):
		nextToObj=self.getObjFromCell(self.you.nextTo)
		self.printDebug('doAction nextTo'+nextToObj.__class__.__name__)
		nextToName=nextToObj.__class__.__name__
		if nextToName=='Statue':
			self.readMessage(nextToObj, self.comboActive and self.combo.name=='read')
		elif nextToName=='DoorRed':
			self.openDoor(nextToObj, self.comboActive and self.combo.name=='keyred')
		elif nextToName=='DoorYellow':
			self.openDoor(nextToObj, self.comboActive and self.combo.name=='keyyellow')
		elif nextToName=='DoorHell':
			self.openDoorHell(nextToObj, self.comboActive and self.combo.name=='keyhell')	
		elif nextToName=='Hell':
			self.currentRoom=[9,9]
			self.generateRoom(self.currentRoom)
		elif nextToName=='Amaltea':
			self.milkAmaltea(self.comboActive and self.combo.name=='milk')
		elif nextToName=='Charon':
			self.payCharon(self.comboActive and self.combo.name=='money')					
		elif nextToName=='Owl':
			self.freeOwl(nextToObj, self.comboActive and self.combo.name=='freeowl')
		elif nextToName=='dark':
			self.popMessage('You need some fire to continue...')

		
		if self.comboActive:	
			if self.combo.name=='sword':
				self.you.changeAction('sword')
				if issubclass(nextToObj.__class__,customObject.Enemy):
					self.hitObj(nextToObj,2)
					
			elif self.combo.name=='speed':
				self.yourSpeed=[c/5 for c in self.cellSize]
				

			if self.combo.name in ['fireblack','firewhite','fireyellow']:
				firePow=['fireblack','firewhite','fireyellow'].index(self.combo.name)+1
				if len(self.yourBullets)<firePow:
					self.yourBullets.append(self.addObject(customObject.Bullet(ftg_frames.getImageFrames('obj_bullets.png'),[c/firePow for c in self.cellSize],self.you.rect.center,self.combo.name,self.you.lastSpeed,4-firePow)))
	
	def openDoor(self, nextToObj, haveKey):
		if haveKey:
			nextToObj.open()
		else:
			self.popMessage('You need a key to open this door')
			
	def openDoorHell(self, nextToObj, haveKey):
		if haveKey:
			nextToObj.block=False
			self.doorHell.open()
		else:
			self.popMessage('You need a key to open this door')
	
	
	def freeOwl(self, nextToObj, ableToFree):
		if ableToFree:
			if self.currentRoom not in self.inventory['owl']:
				nextToObj.changeAnimation('free')
				self.inventory['owl'].append(self.currentRoom)
			self.popMessage('You are free now!')
		else:
			self.popMessage('Can you please free me ?')
				
	
	def popMessage(self,message,greek=False,ableToRead=False):
		self.reading=self.addObject(customObject.Message(ftg_frames.getImageFrames('obj_toolbar.png'),message,pygame.Color('Black'),self.screenSize,greek,ableToRead))
	
	def milkAmaltea(self, hasMilk):
		if hasMilk:
			self.yourLife=9
		else:
			self.popMessage('Do you have a bucket ?')

	def payCharon(self, haveMoney):
		if haveMoney:
			if self.you.rect.centery<=self.screenSize[1]/2:
				self.you.rect.center=[self.boardLimits[0][0]+(8+0.5)*self.cellSize[1],self.boardLimits[0][1]+(8+0.5)*self.cellSize[0]]
			else:
				self.you.rect.center=[self.boardLimits[0][0]+(2+0.5)*self.cellSize[1],self.boardLimits[0][1]+(1+0.5)*self.cellSize[0]]
		else:
			self.popMessage('Have you ever played Charon\'s Obol ?')
		
		
	def readMessage(self, nextToObj, ableToRead):
		self.popMessage(str(nextToObj.reading), nextToObj.greek, ableToRead)
			
		
	def shiftRoom(self, ydelta, xdelta):
		self.currentRoom[0]+=ydelta
		self.currentRoom[1]+=xdelta
		if xdelta<>0: self.you.rect.centerx=(xdelta>0)*(self.boardLimits[0][0]+self.cellSize[0]/2)+(xdelta<0)*(self.boardLimits[1][0]-self.cellSize[0]/2)
		if ydelta<>0: self.you.rect.centery=(ydelta>0)*(self.boardLimits[0][1]+self.cellSize[1]/2)+(ydelta<0)*(self.boardLimits[1][1]-self.cellSize[1]/2)
		#self.printDebug('SHIFT',self.you.rect.center,(self.you.rect.centery-self.boardLimits[0][1])/self.cellSize[1],(self.you.rect.centery-self.boardLimits[0][1])/self.cellSize[1]+self.currentRoom[1]*self.numOfCells[1])
		self.generateRoom(self.currentRoom)
		
	



	def delCell(self, coord):
		self.mapCells[coord[0]][coord[1]].block=False
		self.delObject(self.mapCells[coord[0]][coord[1]])
		self.mapCells[coord[0]][coord[1]]=customObject.CellEmpty()	

	def delCellObj(self, obj):
		self.mapCells[coord[0]][coord[1]].block=False
		self.delObject(self.mapCells[coord[0]][coord[1]])
		self.mapCells[coord[0]][coord[1]]=customObject.CellEmpty()			
	
	def addLife(self, delta):
		self.you.life=min(self.you.life+delta,9)
		if self.you.life<0:
			1/0
		self.lifeText.editText(str(self.you.life))
			
		
	def customUpdate(self):
		if self.pauseScreen==None and self.resultScreen==None and self.loseScreen==None:
			self.clock+=1
			'''
			for c in self.mapCells:
				if self.you.collision_mask.overlap_mask(c.collision_mask,[0,0]).count()>0:
					print 'Bush collision'
			'''
			#you logic
			if False:
				pass
			elif self.you.getCenterX()<=self.boardLimits[0][0]:
				self.printDebug('Out left')
				self.shiftRoom(0,-1)
			elif self.you.getCenterX()>=self.boardLimits[1][0]:
				self.printDebug('Out right')
				self.shiftRoom(0,1)
			elif self.you.getCenterY()<=self.boardLimits[0][1]:
				self.printDebug('Out up')
				self.shiftRoom(-1,0)
			elif self.you.getCenterY()>=self.boardLimits[1][1]:
				self.printDebug('Out down')
				self.shiftRoom(1,0)
			else:
				#myPos=[(self.you.rect.centerx-self.boardLimits[0][0])/self.cellSize[0]+self.currentRoom[0]*self.numOfCells[0],(self.you.rect.centery-self.boardLimits[0][1])/self.cellSize[1]+self.currentRoom[1]*self.numOfCells[1]]
				#myPos=[(self.you.rect.centerx-self.boardLimits[0][0])/self.cellSize[0],(self.you.rect.centery-self.boardLimits[0][1])/self.cellSize[1]]
				myPos,nextPos=self.getCellFromPos(self.you)
				#myPos=[(self.you.getCenterY()-self.boardLimits[0][1])/self.cellSize[1],(self.you.getCenterX()-self.boardLimits[0][0])/self.cellSize[0]]
				#print myPos
				try:
					collideCell=self.getObjFromCell(myPos)
					if collideCell.__class__<>customObject.CellEmpty:
						if collideCell.block and not self.debugOn:
							self.you.stop(myPos)
						if collideCell.collect:
							found=collideCell.name
							self.printDebug('element {name} taken'.format(name=found))
							if found in ELEMENTS:
								#self.popMessage('You receive '+found+' from '+MUSES[ELEMENTS.index(found)])
								self.inventory['elements'][ELEMENTS.index(found)]=True
								self.addObject(customObject.ComboElement(ftg_frames.getImageFrames('obj_elements.png'),self.cellSize,found))
								self.inventory['yourPos']=[self.currentRoom,myPos]
								self.printDebug('room'+str(self.currentRoom)+' cell'+str(myPos)+str([self.you.getCenterX,self.you.getCenterY]),True)
								ftg_dataloader.saveGameOnFile(self.inventory)								
							elif found in ('pot','potfake'):
								self.addLife(collideCell.life)
							self.delCell(myPos)
							#self.delObject(self.mapCells[myPos[0]][myPos[1]])
							#self.mapCells[myPos[0]][myPos[1]]=customObject.CellEmpty()
						if collideCell.hurts:
							self.hitObj(self.you, collideCell.damage)
						elif collideCell.test:
							self.printDebug(myPos,self.you.getCenterX(),self.you.getCenterY())
				except IndexError:
					#printDebug('rect.center',self.you.rect.center,'room',self.currentRoom,'myPos',myPos,'mapCells',len(self.mapCells[0]),len(self.mapCells),True)
					raise

			#clean deaths
			for c in self.dying:
				if c.life==0 and c.hit==0:
					c.changeAnimation('death',1,True,True)
					c.hit=int(2*FPS)
				elif c.life<0:
					if c.__class__.__name__=='Ade':
						self.jumpToLevel(YOUWIN)
					else:
						self.delCell(self.getCellFromPos(c)[0])
						self.dying.remove(c)
			
			#bullet logic
			for b in self.yourBullets:
				#bPosX,bPosY=[(b.rect.centerx-self.boardLimits[0][0])/self.cellSize[0],(b.rect.centerx-self.boardLimits[0][1])/self.cellSize[1]]
				bPos=self.getCellFromPos(b)[0]
				#collideCell=self.mapCells[bPosY][bPosX]
				collideCell=self.getObjFromCell(bPos)
				if collideCell.__class__<>customObject.CellEmpty:
					if issubclass(collideCell.__class__,customObject.Enemy):
						self.hitObj(collideCell,b.damage)
						self.yourBullets.remove(b)
						self.delObject(b)
					elif collideCell.block:
						self.yourBullets.remove(b)
						self.delObject(b)
						
			#enemies
			enemies=[e for l in self.mapCells for e in l if issubclass(e.__class__,customObject.Enemy)]
			if self.clock%(1*FPS)==0 and len(enemies)>0:
				e=random.choice(enemies)
				if e.fire==True:
					e.attackFire(self.you.rect.center)
				else:
					e.attackHand(self.you.rect.center)
				

	def hitObj(self, obj, delta):
		if obj.hit==0:
			self.printDebug(obj.__class__.__name__+str(obj.life),True)
			obj.changeAnimation('hit',1,True,True)
			obj.hit=int(2*FPS)

			obj.lastDamage=delta
			if obj.life-obj.lastDamage<=0:
				self.dying.append(obj)
			if obj==self.you:
				self.you.life-=delta
				self.lifeText.editText(str(self.you.life))
				if self.you.life<=0:
					self.jumpToLevel(GAMEOVER)
			
	
	