import math, random
from pygame.locals import *

import ftg_object
from pw16_variable import *

class You(ftg_object.GameObject):  
	def __init__(self, images, size, centerPosition, boardSize):
		ftg_object.GameObject.__init__(self, images, size, centerPosition)
		self.direction=random.randrange(8)
		self.speed=[0,0]
		self.lastSpeed=self.speed
		self.nextTo=None
		self.action='default'
		self.hit=0
		self.life=9
		self.lastDamage=0

	
	def getCenterX(self):
		return self.rect.centerx#-self.rect.width/2#-self.rect.width/4+int(math.copysign(1,self.speed[0]))*self.rect.width/4

	def getCenterY(self):
		return self.rect.centery#+self.rect.height/4
	
	def stop(self, coord):
		if self.speed<>[0,0]:
			self.lastSpeed=self.speed
			self.nextTo=coord
			self.move(0,0)
		
	def move(self,x_delta,y_delta):
		self.speed=[x_delta,y_delta]
		if x_delta==0 and y_delta==0:
			#self.changeAnimation('default')
			pass
		else:
			self.nextTo=None
			self.changeAnimationDir(x_delta, y_delta)
			'''
			dir=''
			if x_delta<>0: dir=['','l','r'][x_delta/abs(x_delta)]
			if y_delta<>0: dir=[dir,'d','u'][y_delta/abs(y_delta)]
			#dir=[dir,'d','u'][int(math.copysign(1*abs(y_delta),y_delta))]
			self.changeAnimation(self.action+dir)
			'''
	
	def changeAnimationDir(self, x_delta, y_delta):
		dir=''
		if x_delta<>0: dir=['','l','r'][x_delta/abs(x_delta)]
		if y_delta<>0: dir=[dir,'d','u'][y_delta/abs(y_delta)]	
		self.changeAnimation(self.action+dir)	
	
	def changeAction(self, action):
		#lastdir=self.action[-1]
		self.action=action
		#self.changeAnimation(self.action+lastdir)
		self.dirty=1

	
	def updatePosition(self):
		if self.speed<>[0,0]:
			self.rect.left+=self.speed[0]
			self.rect.top+=self.speed[1]
			self.dirty=1
		if self.hit>0:
			self.hit-=1

class Message(ftg_object.GameObject):
	def __init__(self, images, text, color, screenSize, greek=False, ableToRead=False):
		if not greek or  ableToRead:
			font=SMALL_CUSTOMFONT
		else:
			font=GREEK_FONT
			text=''.join(random.sample(text,len(text)))
		height=2*font.get_linesize() 
		self.font=font.render(text, True, color).convert_alpha()	
		ftg_object.GameObject.__init__(self, images, [screenSize[0],height], [screenSize[0]/2,screenSize[1]-height/2])
		#self.images['default'][0].blit(font, (0,0))
		#self.image.blit(self.font, (0,0))
		#self.dirty=1
	
	def updatePosition(self):
		self.image.blit(self.font, (10,10))
		self.dirty=1
		
		
class MouseCursor(ftg_object.GameObject):
	def __init__(self, images, size, centerPosition):
		ftg_object.GameObject.__init__(self, images, size, centerPosition)
		self.dirty = 2
		
	def updatePosition(self):
		self.rect.topleft=pygame.mouse.get_pos()

	
	def click(self, rect):
		collisionRect=pygame.Rect([self.rect.left+self.size[0]/8,self.rect.top-self.size[1]/8],[self.size[0]/4,self.size[1]/4])
		return collisionRect.colliderect(rect)		

class Ring(ftg_object.GameObject):
	def __init__(self, images, size, centerPosition):
		ftg_object.GameObject.__init__(self, images, size, centerPosition)
		
class ComboElement(ftg_object.GameObject):
	def __init__(self, images, size, name):
		self.name=name
		elementNumber=ELEMENTS.index(self.name)
		centerPosition=[(elementNumber+1)*100,50]
		centerPosition=[20+(elementNumber+1+elementNumber/4)*80,50]
		ftg_object.GameObject.__init__(self, images, size, centerPosition, str(elementNumber+1))
		
class Combo(ftg_object.GameObject):
	def __init__(self, images, size, centerPosition, name, firstFrame):
		self.name=name
		ftg_object.GameObject.__init__(self, images, size, centerPosition, firstFrame)
		
class Room(ftg_object.GameObject):
	def __init__(self, images, size, centerPosition):
		ftg_object.GameObject.__init__(self, images, size, centerPosition)
		
	def getAlpha(self):
		pass

class Toolbar(ftg_object.GameObject):
	def __init__(self, images, size, centerPosition):
		ftg_object.GameObject.__init__(self, images, size, centerPosition)	

class Heart(ftg_object.GameObject):
	def __init__(self, images, size, centerPosition):
		ftg_object.GameObject.__init__(self, images, size, centerPosition)
		
		
class RoomObject(ftg_object.GameObject):
	def __init__(self, images, size, centerPosition, firstFrame='default'):
		ftg_object.GameObject.__init__(self, images, size, centerPosition, firstFrame)
		self.block=False
		self.test=False
		self.collect=False
		self.hurts=False

		
class DoorRed(RoomObject):
	def __init__(self, images, size, centerPosition, color):
		self.color=color
		firstFrame='close'+self.color
		RoomObject.__init__(self, images, size, centerPosition, firstFrame)
		self.block=True
	
	def open(self):
		self.block=False
		self.changeAnimation('open'+self.color)
		self.dirty=1
		
		
class DoorYellow(RoomObject):
	def __init__(self, images, size, centerPosition, color):
		self.color=color
		firstFrame='close'+self.color
		RoomObject.__init__(self, images, size, centerPosition, firstFrame)
		self.block=True
	
	def open(self):
		self.block=False
		self.changeAnimation('open'+self.color)
		self.dirty=1		

class DoorHellImage(RoomObject):
	def __init__(self, images, size, centerPosition, color):
		self.color=color
		firstFrame='close'+self.color
		RoomObject.__init__(self, images, size, centerPosition, firstFrame)
		self.block=True
	
	def open(self):
		self.block=False
		self.changeAnimation('open'+self.color)
		self.dirty=1			
		
		
class Bullet(RoomObject):
	def __init__(self, images, size, centerPosition, firstFrame, speed, damage):
		self.speed=speed
		self.damage=damage
		RoomObject.__init__(self, images, size, centerPosition, firstFrame)

	def getCenterX(self):
		return self.rect.centerx
	def getCenterY(self):
		return self.rect.centery		
		
	def updatePosition(self):
		if self.speed<>[0,0]:
			self.rect.left+=self.speed[0]
			self.rect.top+=self.speed[1]
			self.dirty=1
		
class Amaltea(RoomObject):
	def __init__(self, images, size, centerPosition):
		RoomObject.__init__(self, images, size, centerPosition)
		self.name='Amaltea'
		self.block=True
		
class Enemy(RoomObject):
	def __init__(self, images, size, centerPosition):
		RoomObject.__init__(self, images, size, centerPosition)
		self.hurts=True
		self.block=True
		self.damage=1
		self.hit=0
		self.life=1
		self.lastDamage=0
		self.fire=True
		self.targetPos=self.rect.center
	
	def attackFire(self, targetPos):
		self.targetPos=targetPos
		if 'attack' in self.images.keys():
			if self.rect.center<>self.targetPos:
				self.changeAnimation('attack',-1,True,True)
			else:
				self.changeAnimation('default',-1,True,True)
			
	def attackHand(self, targetPos):
		pass
		
	def updatePosition(self):
		if self.hit>0:
			self.hit-=1
			if self.hit<5:
				self.life-=self.lastDamage
				print self,self.life
				self.hit=0
				self.changeAnimation('default',-1,True,True)
		if self.fire:
			if abs(self.targetPos[0]-self.rect.centerx)>5:
				self.rect.left=self.rect.left+math.copysign(1,(self.targetPos[0]-self.rect.centerx))
			if abs(self.targetPos[1]-self.rect.centery)>5:
				self.rect.top=self.rect.top+math.copysign(1,(self.targetPos[1]-self.rect.centery))		

	def getCenterX(self):
		return self.rect.centerx
	def getCenterY(self):
		return self.rect.centery	
			

class Cupido(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)
		self.life=1000
		self.damage=1
		self.fire=False

class Ecatonchiri(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)
		self.life=10
		self.damage=2	
		self.fire=True

class Flower(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)	
		self.life=1
		self.damage=1		

class Soul(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)
		self.life=1
		self.damage=1			
		
class Cerbero(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)
		self.life=15
		self.damage=2	
		self.fire=True

class Arpia(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)
		self.life=1
		self.damage=1			
		
class Centauro(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)
		self.life=5
		self.damage=1			

class Lotofago(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)	
		self.life=1
		self.damage=0		
		
class Sirena(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)
		self.target=None
		self.life=1
		self.damage=1		
	

class Satiro(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)	
		self.life=1
		self.damage=1			
		
class Cavemonster(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)	
		self.life=1
		self.damage=1			
		
class Tip(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)
		self.life=1000
		self.damage=1	
		self.block=False
		self.fire=False

class Bat(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)	
		self.life=1
		self.damage=1		

class Minotauro(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)	
		self.life=5
		self.damage=2
		
		
class Ade(Enemy):
	def __init__(self, images, size, centerPosition):
		Enemy.__init__(self, images, size, centerPosition)	
		self.life=20
		self.damage=5	
		self.fire=True

		
class Element(RoomObject):
	def __init__(self, images, size, centerPosition, firstFrame):
		RoomObject.__init__(self, images, size, centerPosition, firstFrame)
		self.collect=True
		self.name=ELEMENTS[int(firstFrame)-1]


class Bubble(RoomObject):
	def __init__(self, images, size, centerPosition):
		RoomObject.__init__(self, images, size, centerPosition)		

	
		
class Test(RoomObject):
	def __init__(self, images, size, centerPosition):
		RoomObject.__init__(self, images, size, centerPosition)	
		self.test=True
		
class Owl(RoomObject):
	def __init__(self, images, size, centerPosition, free):
		firstFrame='default'
		if free:
			firstFrame='gone'
		RoomObject.__init__(self, images, size, centerPosition, firstFrame)
		self.name='owl'
		self.block=True
		
class Pot(RoomObject):
	def __init__(self, images, size, centerPosition):
		RoomObject.__init__(self, images, size, centerPosition)
		self.collect=True
		self.name='pot'
		self.life=1

class PotFake(RoomObject):
	def __init__(self, images, size, centerPosition):
		RoomObject.__init__(self, images, size, centerPosition)
		self.collect=True
		self.name='potfake'
		self.life=-1
		
class Bush(RoomObject):
	def __init__(self, images, size, centerPosition):
		firstFrame='2'
		if random.random()>0.8: firstFrame=str(random.randrange(8))
		RoomObject.__init__(self, images, size, centerPosition, firstFrame)
		self.name='bush'
		self.block=True

class Tree(RoomObject):
	def __init__(self, images, size, centerPosition):
		firstFrame=str(random.randrange(5))
		RoomObject.__init__(self, images, size, centerPosition, firstFrame)
		self.name='tree'
		self.block=True		
		
class Rock(RoomObject):
	def __init__(self, images, size, centerPosition):
		firstFrame=str(random.choice([0,2,4,6]))
		if random.random()>0.8: firstFrame=str(random.choice([1,3,5,7]))
		RoomObject.__init__(self, images, size, centerPosition, firstFrame)
		self.name='rock'
		self.block=True		
		
class Statue(RoomObject):
	def __init__(self, images, size, centerPosition, room):
		firstFrame=str(random.randrange(8))
		RoomObject.__init__(self, images, size, centerPosition, firstFrame)
		self.name='statue'
		self.greek=READING[room[1]][room[0]][0]
		self.reading=' '+READING[room[1]][room[0]][1]
		self.block=True		

class Torch(RoomObject):
	def __init__(self, images, size, centerPosition):
		RoomObject.__init__(self, images, size, centerPosition)
		self.name='torch'
		self.block=True			
	
	
class CellEmpty():
	def __init__(self):
		self.name='empty'
		self.block=False
		self.test=False
		self.collect=False
		self.hurts=False
		self.dirty=0	
	
class CellBlock():
	def __init__(self):
		self.name='block'
		self.block=True
		self.test=False
		self.collect=False
		self.hurts=False
		self.dirty=0	

class CellDark():
	def __init__(self):
		self.name='dark'
		self.block=True
		self.test=False
		self.collect=False
		self.hurts=False
		self.dirty=0		

class Charon():
	def __init__(self):
		self.name='Charon'
		self.block=True
		self.test=False
		self.collect=False
		self.hurts=False
		self.dirty=0					

class DoorHell():
	def __init__(self):
		self.name='DoorHell'
		self.block=True
		self.test=False
		self.collect=False
		self.hurts=False
		self.dirty=0			
		
class Hell():
	def __init__(self):
		self.name='Hell'
		self.block=True
		self.test=False
		self.collect=False
		self.hurts=False
		self.dirty=0	
		

		
class PauseScreen(ftg_object.GameObject):
	def __init__(self, images, screenSize):
		print('pause screen created')
		ftg_object.GameObject.__init__(self, images, screenSize, [sc/2 for sc in screenSize])
